/*
* Dateiname.... lowLevel.c
* erstellt am.. 2013-5-25
* gendert am.. 2014-03-08
* Autor........ Clemens B Fahr
* email........ clemens.fahr[at]fahrnet.de
* Kommentar.... based on RFM_wireless_small_base_01 SVN 465
* ............. remove the LED2 feature, remove the RFM_receive routine due to switching to ISR
* ............. improve the LED toggle function and add inhibit by one timer cylce befor next blink is possible
* ............. 
*/

// includes
//------------------------------------------------------------------
#include "lowlevel.h"
#include "global.h"

void LOWLEVEL_init_all(void)
{	
	// write oscillator calibration byte
	//OSCCAL = 0x4C; 	// has to be determined by AVR Studio
	DDRA = 0;
	DDRB = 0;
	
	// clock prescaler
	CLKPR = (0x01 << CLKPCE);	// enable to write the prescaler
	CLKPR = CLK_PRESCALER;		// write the prescaler value
	
	// start basic initialisation of data direction, port setting and pull-up
	LED1_DDR;
        LED1_status = ON;
        LED1(LED1_status);
        LED1_blink_count = 0;
        LED1_enabled = TRUE;
	
	UART_TX(HIGH);
	UART_TX_DDR;
	DEBUGPORTLEVEL(HIGH);
	DEBUG_DDR;
	
	LOWLEVEL_delay_1ms(200);
    LOWLEVEL_init_RFM();	// init the RFM01 module
}

void LOWLEVEL_init_RFM(void)
{
	// init RFM pins and their data direction, init pin level
	RFM_SEL(HIGH);
	RFM_SDI(HIGH);
	RFM_SCK(LOW);
	RFM_SEL_DDR;
	RFM_SDI_DDR;
	RFM_SDO_DDR;
	RFM_SCK_DDR;
	RFM_IRQ_DDR;
    RFM_IRQ_CLEAR;
    RFM_IRQ_ENABLE;
	
	send_command_RFM(RFM_CMD_READSTAT);
	send_command_RFM(RFM_CMD_CONFIGSET);
	send_command_RFM(RFM_CMD_FREQSET);
	send_command_RFM(RFM_CMD_DATARATE);
	send_command_RFM(RFM_CMD_AFCSET);
	send_command_RFM(RFM_CMD_DATAFLT);
	//send_command_RFM(RFM_CMD_LBDCLKDIV);
	send_command_RFM(RFM_CMD_RX_OFF);
	//send_command_RFM(RFM_CMD_WKPTIMER);
	send_command_RFM(RFM_CMD_FIFO_DIS);
	send_command_RFM(RFM_CMD_FIFO_SYNC);
	send_command_RFM(RFM_CMD_RX_ON);
        
    // clear and init the complete RX-Buffer
	for (numberReceivedBytes=0;numberReceivedBytes<MAX_BUFFER_SIZE;numberReceivedBytes++) {
		RX_buffer[numberReceivedBytes] = 0xFF;
	}
    numberReceivedBytes = 0;
    telegramComplete = FALSE;
}

uint16_t send_command_RFM(uint16_t aCommand)
{
	uint8_t i;
	uint16_t temp=0;
	
	RFM_SCK(LOW);
	RFM_SEL(LOW);
	for (i=0;i<16;i++) {
		temp <<= 1;
		if (RFM_SDO_HIGH) {
			temp |= 0x0001;
		}
		if (aCommand & 0x8000) {
			RFM_SDI(HIGH);
		}
		else {
			RFM_SDI(LOW);
		}
		RFM_SCK(HIGH);
		RFM_SCK(LOW);
		aCommand <<= 1;
	}
	RFM_SEL(HIGH);
	nop();
	nop();
	return temp;
}

void LOWLEVEL_UART_TX(uint8_t aByte)
{
	uint8_t i;
	
	DEBUGPORTLEVEL(LOW);	
	// send the start bit
	UART_TX(LOW);
	UART_bit_delay();
	// send the aByte payload
	for (i=0;i<8;i++) {
		if (aByte & 0x01) {
			UART_TX(HIGH);
		}
		else {
			UART_TX(LOW);
		}
		UART_bit_delay();
		aByte >>= 1;
	}
	// send the stop bit
	UART_TX(HIGH);
	UART_bit_delay();
	DEBUGPORTLEVEL(HIGH);
	UART_bit_delay();
	UART_bit_delay();
}

void LOWLEVEL_RFM_stop_RX(void)
{
	nop();
	send_command_RFM(RFM_CMD_RX_OFF);
	nop();
	nop();
	nop();
	nop();
	nop();
}

void UART_bit_delay(void)
{
	run_timer0(PRELOAD_BIT_T0,CLK_SRC_BIT_T0,OFF);
	do
	{	//while timer does not overflow stay here = delay
	}
	while ((TIFR & (0x01 << TOV0)) == 0);
	stop_timer0();
}

// some default timer routines
void LOWLEVEL_delay_1ms(uint16_t multiplier)
{
	// produces a (1 * multiplier)ms delay
	uint16_t i;
	
	for (i=0;i<multiplier;i++) {
		run_timer1(PRELOAD_1MS_T1,CLK_SRC_1MS_T1,OFF);
		do
		{	//while timer does not overflow stay here = delay
		}
		while ((TIFR & (0x01 << TOV1)) == 0);
		stop_timer1();
	}
}

void run_timer0(uint8_t preloadvalue, uint8_t clocksource, uint8_t IRQ_enabled)
{
	// used for the software UART only
	// clear interrupt flag TOV0
	TIFR = (0x01 << TOV0); // annotation: remember to NOT use the bit-"OR"!! Interrupt flags are cleared this way ONLY!!
	// preload timer counter and the compare match register for 8bit counter mode
	TCNT0L = preloadvalue;
	OCR0A = 0xFF;
	//TCNT0H = 0xFF;
	// setup timer/counter 1 control register
	TCCR0B |= clocksource;
	if ( IRQ_enabled ) {
		TIMSK |= (0x01 << TOIE0);
	}
	else {
		TIMSK &= ~(0x01 << TOIE0);
	}
}

void stop_timer0(void)
{
	// setup timer/counter 1 control register, stop clocking of timer
	// makes the timer stop
	TCCR0B &= TIMER0_CLK_OFF;
	// disables timer interrup
	TIMSK &= ~(0x01 << TOIE0);
	// preload timer counter to bottom
	TCNT0L = 0x00;
	OCR0A = 0x00;
	//TCNT0H = 0x00;
}

void run_timer1(uint8_t preloadvalue, uint8_t clocksource, uint8_t IRQ_enabled)
{
	// clear interrupt flag TOV1
	TIFR = (0x01 << TOV1); // annotation: remember to NOT use the bit-"OR"!! Interrupt flags are cleared this way ONLY!!
	// preload timer counter and the compare match register for 8bit counter mode
	OCR1C = 0xFF; // not sure, if this is truely needed
	TCNT1 = preloadvalue;
	// setup timer/counter 1 control register
	TCCR1B |= clocksource;
	if ( IRQ_enabled ) {
		TIMSK |= (0x01 << TOIE1);
	}
	else {
		TIMSK &= ~(0x01 << TOIE1);
	}
}

void stop_timer1(void)
{
	// setup timer/counter 1 control register, stop clocking of timer
	// makes the timer stop
	TCCR1B &= TIMER1_CLK_OFF;
	// disables timer interrup
	TIMSK &= ~(0x01 << TOIE1);
	// preload timer counter to bottom
	TCNT1 = 0x00;
}

void toggle_LED1(void)
{
        if ( LED1_status == ON ) {
                LED1_status = OFF;
        }
        else {
                LED1_status = ON;
        }
        LED1(LED1_status);
}

void LOWLEVEL_toggle_LED1(uint8_t inhibit)
{
        if ( LED1_enabled & !inhibit) {
                LED1_enabled = FALSE;
                toggle_LED1();
                LED1_blink_count++;
                // start timer1 to run in interrupt mode
                run_timer1(PRELOAD_500MS_T1, CLK_SRC_500MS_T1, ON);
        }
        if ( !LED1_enabled & (LED1_blink_count <= LED1_MAX_BLINK) & inhibit ) {
                toggle_LED1();
                LED1_blink_count++;
                run_timer1(PRELOAD_500MS_T1, CLK_SRC_500MS_T1, ON);
        }
        else if ( (LED1_blink_count-1) == LED1_MAX_BLINK ) {
                toggle_LED1();
                run_timer1(PRELOAD_500MS_T1, CLK_SRC_500MS_T1, ON);
                LED1_blink_count++;
        }
        else if ( (LED1_blink_count-2) == LED1_MAX_BLINK ) {
                // inhibit by one further timer cylce befor next blink is possible
                stop_timer1();
                LED1_blink_count = 0;
                LED1_enabled = TRUE;
        }
}

